
/* 
* This file is part of VL53L1 Platform 
* 
* Copyright (c) 2016, STMicroelectronics - All Rights Reserved 
* 
* License terms: BSD 3-clause "New" or "Revised" License. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are met: 
* 
* 1. Redistributions of source code must retain the above copyright notice, this 
* list of conditions and the following disclaimer. 
* 
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation 
* and/or other materials provided with the distribution. 
* 
* 3. Neither the name of the copyright holder nor the names of its contributors 
* may be used to endorse or promote products derived from this software 
* without specific prior written permission. 
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
* 
*/

#include "vl53l1_platform.h"
#include <string.h>
#include <time.h>
#include <math.h>

#define VL53L1_ERROR_NONE                              (0)
#define VL53L1_ERROR_CONTROL_INTERFACE                 (- 13)

#define I2C_CFG_MONCLKSTR			(1 << 4)
static LPC_I2C_T *s_pI2Cx = NULL;

int8_t VL53L1_CommsInitialise(LPC_I2C_T *pI2Cx, TIICMode fm) {
  int8_t result = 1;
  s_pI2Cx = pI2Cx;
  if (s_pI2Cx != NULL) {
    result = IIC_Init (pI2Cx, fm) ? VL53L1_ERROR_NONE : VL53L1_ERROR_CONTROL_INTERFACE;
    s_pI2Cx->CFG |= I2C_CFG_MONCLKSTR;
  }
  return result;
}

int8_t VL53L1_CommsClose(LPC_I2C_T *pI2Cx) {
  if (pI2Cx != NULL) IIC_Reset (pI2Cx);
  s_pI2Cx = NULL;
  return 1;
}

static const TIICDataSpec _ds = {IIC_Reg16, IIC_Data8, IIC_BigEndian};

#if 1
int8_t VL53L1_WriteMulti( uint16_t dev, uint16_t index, uint8_t *pdata, uint32_t count) {
  uint16_t adr = index;
  return IIC_Write (s_pI2Cx, dev, &adr, pdata, count, &_ds, NULL) ? VL53L1_ERROR_NONE : VL53L1_ERROR_CONTROL_INTERFACE;
}

int8_t VL53L1_ReadMulti(uint16_t dev, uint16_t index, uint8_t *pdata, uint32_t count){
  uint16_t adr = index;
  return IIC_Read (s_pI2Cx, dev, &adr, pdata, count, &_ds, NULL) ? VL53L1_ERROR_NONE : VL53L1_ERROR_CONTROL_INTERFACE;
}

int8_t VL53L1_WrByte(uint16_t dev, uint16_t index, uint8_t data) {
  return VL53L1_WriteMulti(dev, index, &data, 1);
}

int8_t VL53L1_WrWord(uint16_t dev, uint16_t index, uint16_t data) {
  uint8_t buffer[2];
  *(uint16_t *)(void *)&buffer = __builtin_bswap16 (data);
//  if (index % 2 == 1) {
//    VL53L1_WriteMulti(dev, index, &buffer[0], 1);
//    return VL53L1_WriteMulti(dev, index + 1, &buffer[1], 1);
//  } else {
    return VL53L1_WriteMulti(dev, index, buffer, 2);
//  }
}

int8_t VL53L1_WrDWord(uint16_t dev, uint16_t index, uint32_t data) {
  uint8_t  buffer[4];
  *(uint32_t *)(void *)&buffer = __builtin_bswap32 (data);
  return VL53L1_WriteMulti(dev, index, buffer, 4);
}

int8_t VL53L1_RdByte(uint16_t dev, uint16_t index, uint8_t *data) {
  return VL53L1_ReadMulti(dev, index, data, 1);
}

int8_t VL53L1_RdWord(uint16_t dev, uint16_t index, uint16_t *data) {
  const TIICDataSpec _dsw = {IIC_Reg16, IIC_Data16, IIC_BigEndian};
  uint16_t adr = index;
  return IIC_Read (s_pI2Cx, dev, &adr, (void *)data, 2, &_dsw, NULL) ? VL53L1_ERROR_NONE : VL53L1_ERROR_CONTROL_INTERFACE;
}

int8_t VL53L1_RdDWord(uint16_t dev, uint16_t index, uint32_t *data) {
  const TIICDataSpec _dsd = {IIC_Reg16, IIC_Data32, IIC_BigEndian};
  uint16_t adr = index;
  return IIC_Read (s_pI2Cx, dev, &adr, (void *)data, 4, &_dsd, NULL) ? VL53L1_ERROR_NONE : VL53L1_ERROR_CONTROL_INTERFACE;
}
#else
int8_t VL53L1_WriteMulti( uint16_t dev, uint16_t index, uint8_t *pdata, uint32_t count) {
	return 0; // to be implemented
}

int8_t VL53L1_ReadMulti(uint16_t dev, uint16_t index, uint8_t *pdata, uint32_t count){
	return 0; // to be implemented
}

int8_t VL53L1_WrByte(uint16_t dev, uint16_t index, uint8_t data) {
	return 0; // to be implemented
}

int8_t VL53L1_WrWord(uint16_t dev, uint16_t index, uint16_t data) {
	return 0; // to be implemented
}

int8_t VL53L1_WrDWord(uint16_t dev, uint16_t index, uint32_t data) {
	return 0; // to be implemented
}

int8_t VL53L1_RdByte(uint16_t dev, uint16_t index, uint8_t *data) {
	return 0; // to be implemented
}

int8_t VL53L1_RdWord(uint16_t dev, uint16_t index, uint16_t *data) {
	return 0; // to be implemented
}

int8_t VL53L1_RdDWord(uint16_t dev, uint16_t index, uint32_t *data) {
	return 0; // to be implemented
}

int8_t VL53L1_WaitMs(uint16_t dev, int32_t wait_ms){
	return 0; // to be implemented
}
#endif
