/*----------------------------------------------------------*
   LPC84x EXTRA - peripheral wrapper library (I2C) v1.0
                                       Last Edit '21 01/15
   Copyright (c) BestTechnology CO.,LTD. 2021
 *----------------------------------------------------------*/

#ifndef _WL_I2C_H_INCLUDE
#define _WL_I2C_H_INCLUDE

#include  <stdint.h>
#include  <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
  #define   __I     volatile             /*!< Defines 'read only' permissions */
#else
  #define   __I     volatile const       /*!< Defines 'read only' permissions */
#endif
#define     __O     volatile             /*!< Defines 'write only' permissions */
#define     __IO    volatile             /*!< Defines 'read / write' permissions */


typedef struct {					/*!< I2C Structure */
  __IO uint32_t  CFG;			  /* 0x00 */
  __IO uint32_t  STAT;
  __IO uint32_t  INTENSET;
  __O  uint32_t  INTENCLR;
  __IO uint32_t  TIMEOUT;		/* 0x10 */
  __IO uint32_t  CLKDIV;
  __IO uint32_t  INTSTAT;
       uint32_t  Reserved0[1];  
  __IO uint32_t  MSTCTL;			  /* 0x20 */
  __IO uint32_t  MSTTIME;
  __IO uint32_t  MSTDAT;
       uint32_t  Reserved1[5];
  __IO uint32_t  SLVCTL;			  /* 0x40 */
  __IO uint32_t  SLVDAT;
  __IO uint32_t  SLVADR0;
  __IO uint32_t  SLVADR1;
  __IO uint32_t  SLVADR2;			  /* 0x50 */
  __IO uint32_t  SLVADR3;
  __IO uint32_t  SLVQUAL0;
       uint32_t  Reserved2[9];
  __I  uint32_t  MONRXDAT;			/* 0x80 */
} LPC_I2C_T;

// [h
typedef enum {
  IIC_Sm,   // Standard-mode 100k
  IIC_Fm,   // Fast-mode 400k
  IIC_Fmp,  // Fast-mode Plus 1M
  IIC_Hsm,  // Hish-speed mode 3.4M
  IIC_UFm   // Ultra Fast-mode 5M
} TIICMode;

// GfBA
typedef enum {
  IIC_BigEndian,
  IIC_LittleEndian,
} TIICEndian;

// WX^AhX
typedef enum {
  IIC_Reg8 = 1,
  IIC_Reg16 = 2,
  IIC_Reg32 = 4,
  IIC_Reg64 = 8,
} TIICRegWidth;

// f[^
typedef enum {
  IIC_Data8 = 1,
  IIC_Data16 = 2,
  IIC_Data32 = 4,
  IIC_Data64 = 8,
} TIICDataWidth;

typedef struct {
  TIICRegWidth  regwidth;
  TIICDataWidth datawidth;
  TIICEndian    endian;
} TIICDataSpec;

// WX^AhXM̒x[u(Kvɉ)
typedef bool (*pIICDelayFunc) (void);

extern void (*IIC_csw) (void);

extern void IIC_Reset (LPC_I2C_T *I2Cx);
extern bool IIC_Freq (LPC_I2C_T *I2Cx, TIICMode m);
extern bool IIC_Init (LPC_I2C_T *I2Cx, TIICMode m);
extern bool IIC_GeneStart (LPC_I2C_T *I2Cx);
extern bool IIC_GeneStop (LPC_I2C_T *I2Cx);

extern bool IIC_Read (LPC_I2C_T *I2Cx, uint8_t SlaveAddr, void *pRegAddr, void *prxd, int length, const TIICDataSpec *pDs, pIICDelayFunc pDf);
extern bool IIC_ReadS (LPC_I2C_T *I2Cx, uint8_t SlaveAddr, void *prxd, int length, const TIICDataSpec *pDs, pIICDelayFunc pDf);
extern bool IIC_Write (LPC_I2C_T *I2Cx, uint8_t SlaveAddr, void *pRegAddr, const void *ptxd, int length, const TIICDataSpec *pDs, pIICDelayFunc pDf);
extern bool IIC_WriteS (LPC_I2C_T *I2Cx, uint8_t SlaveAddr, const void *ptxd, int length, const TIICDataSpec *pDs, pIICDelayFunc pDf);

#define LPC_I2C0_BASE         (0x40050000UL)
#define LPC_I2C1_BASE         (0x40054000UL)
#define LPC_I2C0            ((LPC_I2C_T         *) LPC_I2C0_BASE)
#define LPC_I2C1            ((LPC_I2C_T         *) LPC_I2C1_BASE)

#ifdef __cplusplus
}
#endif

#endif //_WL_I2C_H_INCLUDE
