﻿/*
  DXMIO Dynamixel Client簡易モニタ

  smpl18(dxclient_imu).binを書き込んだDXMIOをPCでモニタ

  ・プロトコルV2限定
  ・GUIのツールを用意するのが面倒だったのでDX2LIBのCUIベースのサンプル
    から流用
  ・テキストIだけでは寂しかったのでちょっとだけグラフィック表示
  ・数値をモニタするだけでは応用に欠けるので一部の設定を変更可
  ・コンパイラオプションのリンクするライブラリにgdi32を追加する事
*/

#include  <stdio.h>
#include  <string.h>
#include  <conio.h>
#include  <io.h>

#define _DYNAMICLOAD
#include  "dx2lib.h"

// マクロ定義
#define _COMPORT    "\\\\.\\COM3"   // ポート名
#define _BAUDRATE   (1000000)       // ボーレート[bps]
#define _TARGET_ID  (200)           // 対象ID

#define CLAMP(x, low, high)  (((x) > (high)) ? (high) : (((x) < (low)) ? (low) : (x)))
#define cls()           printf ("\x1b[2J");
#define gotoxy(xx, yy)  printf ("\x1b[%d;%dH", yy + 1, xx + 1);


// DXMIOのIMU部分の全アイテム
typedef struct {
  int16_t acc_x;    // Acceleration Data
  int16_t acc_y;
  int16_t acc_z;
  int16_t mag_x;    // Magnetometer Data
  int16_t mag_y;
  int16_t mag_z;
  int16_t gyro_x;   // Gyroscope Data
  int16_t gyro_y;
  int16_t gyro_z;
  int16_t heading;  // Euler Data
  int16_t roll;
  int16_t pitch;
  int16_t q_w;      // Quaternion Data
  int16_t q_x;
  int16_t q_y;
  int16_t q_z;
  int16_t lia_x;    // Linear Acceleration Data
  int16_t lia_y;
  int16_t lia_z;
  int16_t grv_x;    // Gravity Vector Data
  int16_t grv_y;
  int16_t grv_z;
  int8_t  temp;     // Temperature
} __attribute__ ((packed)) Tbno055;

// DXMIOのコントロールテーブル
typedef struct {
  uint16_t  ModelNumber;      // モデルナンバー (R)
  uint32_t  ModelInfomation;  // モデル情報 (R)
  uint8_t   FirmwareVersion;  // ファームウェアバージョン (R)
  uint8_t   ID;               // ID (R/W & NVM)
  uint8_t   Baudrate;         // ボーレート (R/W & NVM)
  uint8_t   WriteNVM;         // 不揮発データ書き込みフラグ (R/W)

  uint8_t   LED;              // LED2  (R/W)
  uint8_t   Terminator;       // 終端抵抗 (R/W & NVM)
  uint8_t   PinCfg[12];       // GPIO端子機能設定 (R/W & NVM)
  uint16_t  PWMFrequency;     // PWM周波数設定 (R/W & NVM)
  uint16_t  PWMDuty[6];       // PWMデューティー設定 (R/W & NVM)
  uint8_t   reserve2[10];
  uint16_t  Capture[4];       // パルス設定・計測値 (R/W)
  uint16_t  GPIO_OUT;         // GPIO出力設定値 (R/W)
  uint16_t  GPIO_IN;          // GPIO入力値 (R)
  uint16_t  ADV[12];          // アナログ電圧測定値 (R)
  uint16_t  DAV[12];          // アナログ電圧設定値 (R/W)
  uint8_t   IMU_OprMode;      // IMU動作モード (R/W & NVM)
  uint8_t   IMU_AxisPlace;    // IMUセンサ配置 (R/W & NVM)
  Tbno055   IMU;              // IMU測定値 (R)
  uint8_t   IMU_stat;         // IMUステータス値 (R)
  uint8_t   IMU_err;          // IMUエラー値 (R)
  uint8_t   userarea[20];     //
} __attribute__ ((packed)) TE097BCtrlTable;


// コントロールテーブルの各アイテムのオフセットアドレス
#define _ADDR_ModelNumber     ((int)&((TE097BCtrlTable*)0)->ModelNumber)
#define _ADDR_ModelInfomation ((int)&((TE097BCtrlTable*)0)->ModelInfomation)
#define _ADDR_FirmwareVersion ((int)&((TE097BCtrlTable*)0)->FirmwareVersion)
#define _ADDR_ID              ((int)&((TE097BCtrlTable*)0)->ID)
#define _ADDR_Baudrate        ((int)&((TE097BCtrlTable*)0)->Baudrate)
#define _ADDR_WriteNVM        ((int)&((TE097BCtrlTable*)0)->WriteNVM)
#define _ADDR_LED             ((int)&((TE097BCtrlTable*)0)->LED)
#define _ADDR_Terminator      ((int)&((TE097BCtrlTable*)0)->Terminator)
#define _ADDR_PinCfg          ((int)&((TE097BCtrlTable*)0)->PinCfg)
#define _ADDR_PWMFrequency    ((int)&((TE097BCtrlTable*)0)->PWMFrequency)
#define _ADDR_PWMDuty         ((int)&((TE097BCtrlTable*)0)->PWMDuty)
#define _ADDR_Capture         ((int)&((TE097BCtrlTable*)0)->Capture)
#define _ADDR_OUT             ((int)&((TE097BCtrlTable*)0)->GPIO_OUT)
#define _ADDR_IN              ((int)&((TE097BCtrlTable*)0)->GPIO_IN)
#define _ADDR_ADV             ((int)&((TE097BCtrlTable*)0)->ADV)
#define _ADDR_IMU_OprMode     ((int)&((TE097BCtrlTable*)0)->IMU_OprMode)
#define _ADDR_IMU_AxisPlace   ((int)&((TE097BCtrlTable*)0)->IMU_AxisPlace)
#define _ADDR_IMU             ((int)&((TE097BCtrlTable*)0)->IMU)
#define _ADDR_IMU_stat        ((int)&((TE097BCtrlTable*)0)->IMU_stat)
#define _ADDR_IMU_err         ((int)&((TE097BCtrlTable*)0)->IMU_err)
#define _ADDR_userarea        ((int)&((TE097BCtrlTable*)0)->userarea)

// V2におけるボーレートのテーブル
const uint32_t baudlist[9] = { 9600, 57600, 115200, 1000000, 2000000, 3000000, 4000000, 4500000, 10500000 };

//===========================================================
// 実行時にポート番号・ボーレート・IDを選択
//===========================================================
bool InputNum (int32_t *num) {
  char buf[100];

  fgets (buf, sizeof(buf), stdin);
  rewind (stdin);

  return (strlen(buf) && sscanf(buf, "%d", num) == 1);
}

bool InputStr (char *str) {
  char buf[100];

  fgets (buf, sizeof(buf), stdin);
  rewind (stdin);

  return (strlen(buf) && sscanf(buf, "%s", str) == 1);
}

bool EditComAndBaudAndTarget (char *comport, uint32_t *baud, uint8_t *id) {
  bool result = true;

  int32_t n;

  printf ("Input comport name (default %s) = ", _COMPORT);
  if (!(result = InputStr (comport))) result = strcpy (comport, _COMPORT);

  printf ("Input baudrate (default %dbps) = ", _BAUDRATE);
  if (!InputNum (&n)) { n = _BAUDRATE; result = false; }
  if (n >= 9600 && n <= 4000000) *baud = n; else *baud = _BAUDRATE;

  printf ("Input target id (default %d) = ", _TARGET_ID);
  if (!InputNum (&n)) { n = _TARGET_ID; result = false; }
  if (n >= 0 && n <= 0xfd) *id = n; else *id = _TARGET_ID;

  return result;
}

//===========================================================
// サブルーチン
//===========================================================
bool en_vterm_proc (FILE *stream) {
  HANDLE handle = (HANDLE)_get_osfhandle(_fileno(stream));
  DWORD mode = 0;
  if (!GetConsoleMode(handle, &mode)) return false;
  if (!SetConsoleMode(handle, mode | ENABLE_VIRTUAL_TERMINAL_PROCESSING)) return false;
  return true;
}

bool AreYouDXMIO (TDeviceID dev, uint8_t id) {
  uint16_t modelno;
  if (DX2_ReadWordData (dev, id, _ADDR_ModelNumber, &modelno, NULL)) return modelno == 0x4023;
  return false;
}

void Draw (HWND hwd, HDC hdc, HPEN *pen, HBRUSH *brush, TE097BCtrlTable *table, int *prevx, int *prevy, int x) {
  RECT rect;

  // ウィンドウのサイズを取得
  GetWindowRect (hwd, &rect);
  int w = CLAMP((rect.right - rect.left) - 50, 0, 8191);
  int h = CLAMP((rect.bottom - rect.top) - 50, 0, 8191);

  // GPIOの入力値をBOX描画
  SelectObject (hdc, pen[3]);
  for (int i = 0; i < 12; i++) {
    SelectObject (hdc, brush[(table->GPIO_IN & (1 << (11 - i))) ? 1 : 0]);
    Rectangle (hdc , i * 12 + 10, 30 , i * 12 + 20, 40);
  }

  // IMUのHeadingのみグラフ描画
  if (x % w == 0) {
    InvalidateRect (hwd, NULL , TRUE);
    for (int i = 0; i < 3; i++){
      SelectObject (hdc, pen[i]);
      MoveToEx (hdc, 0, 0, NULL);
      prevy[i] = 0;
    }
    *prevx = 0;
  }
  SelectObject (hdc, pen[0]);
  MoveToEx (hdc, *prevx, prevy[0], NULL);
  LineTo (hdc, x % w, prevy[0] = (h >> 1) - (h >> 1) * (table->IMU.heading / 16.0) / 360.0);

  SelectObject (hdc, pen[1]);
  MoveToEx (hdc, *prevx, prevy[1], NULL);
  LineTo (hdc, x % w, prevy[1] = (h >> 1) - (h >> 1) * (table->IMU.roll / 16.0) / 360.0);

  SelectObject (hdc, pen[2]);
  MoveToEx (hdc, *prevx, prevy[2], NULL);
  LineTo (hdc, x % w, prevy[2] = (h >> 1) - (h >> 1) * (table->IMU.pitch / 16.0) / 360.0);

  *prevx = (x % w);
}

//===========================================================
// メイン
//===========================================================
int main (void) {
  bool term = false;
  int32_t n;
  uint8_t b;

  int x = 0, prevx, prevy[3];

  char        comname[20];
  uint32_t    baud;
  uint8_t     id;

  TDeviceID   dev;
  TErrorCode  err;

  TE097BCtrlTable table;
  uint8_t     led;

  en_vterm_proc (stdout);

  // DX2LIBをロード
  if (LoadDLL ()) {
    // お絵描きのための下準備(GDIオブジェクトの作成等)
    HWND hwd = GetConsoleWindow ();
    HDC hdc = GetDC (hwd);
    HPEN pen[] = {
      CreatePen (PS_SOLID, 0, RGB (255,   0,   0)),
      CreatePen (PS_SOLID, 0, RGB (  0, 255,   0)),
      CreatePen (PS_SOLID, 0, RGB (  0,   0, 255)),
      CreatePen (PS_SOLID, 0, RGB (255, 255, 255)),
    };
    HBRUSH brush[] = {
      CreateSolidBrush (RGB (  0,   0,   0)),
      CreateSolidBrush (RGB (255,   0, 255)),
    };

    // ポート・ID・ボーレートをデフォルト値から変更
    EditComAndBaudAndTarget (comname, &baud, &id);
    printf("\nCOM=%s, Baudrate=%d, id=%d\n", comname, baud, id);

    // ポートオープン
    if ((dev = DX2_OpenPort (comname, baud))) {
      printf ("Successful opening of %s\n", comname);

      // termフラグをtrueにするとループを抜ける
      for (;!term;) {
        // キー入力に応じてDXMIOの設定を変更
        if (kbhit ()) {
          switch (_getch ()) {
            case 's': // 全スキャン
              printf("\n>Scan\n");
              uint32_t num;
              TDx2AlarmStatus stat[253];
              for (int i = 0; i < 9; i++) {
                DX2_SetBaudrate (dev, baudlist[i]);
                printf("\rBaud=%d[bps]         ", baudlist[i]);
                num = 253;
                if (DX2_Ping2 (dev, &num, stat, &err)) {
                  printf ("\n %d device found\n", num);
                  for (int j = 0; j < num; j++) {
                    printf (" Found ID=%d stat:$%04X\n", stat[j].id, stat[j].Status);
                    // もし見つかったらホストのidとbaudrateへ反映
                    if (AreYouDXMIO (dev, stat[j].id)) {
                      id = stat[j].id;
                      baud = baudlist[i];
                    }
                  }
                }
              }
              DX2_SetBaudrate (dev, baud);
              printf ("\n");
              break;
            case 'i': // 対象ID変更
              printf("\n>Set ID");
              printf ("\nInput new host id (current %d) 0..253 = ", id);
              if (InputNum (&n)) {
                if (n >= 0 && n <= 0xfd) {
                  id = n;
                } else printf ("NG\n");
              } else printf ("NG\n");
              break;
            case 'b': // PCのボーレート変更
              printf("\n>Set Baudrate");
              for (n = 0; (n < 9) && (baudlist[n] != baud); n++) ;
              printf ("\nInput new host baudrate (current %d:%dbps) 0..4 = ", n, baudlist[n]);
              if (InputNum (&n)) {
                if ((n >= 0 && n <= 4) /*|| (n == 6)*/) {
                  if (DX2_SetBaudrate (dev, baudlist[n])) baud = baudlist[n];
                } else printf ("NG\n");
              } else printf ("NG\n");
              break;
            case 'I': // DXMIOのID変更
              printf("\n>Set DXMIO ID");
              if (AreYouDXMIO (dev, id)) {
                printf ("\nInput new target id (current %d) 0..253 = ", id);
                if (InputNum (&n)) {
                  if (n >= 0 && n <= 0xfd) {
                    if (DX2_WriteByteData (dev, id, _ADDR_ID, n, &err)) {
                      id = n;
                      printf ("OK\n");
                    } else printf ("NG\n");
                  } else printf ("NG\n");
                } else printf ("NG\n");
              } else printf ("NG\n");
              break;
            case 'B': // DXMIOのボーレート変更
              printf("\n>Set DXMIO Baudrate");
              for (n = 0; (n < 9) && (baudlist[n] != baud); n++) ;
              if (AreYouDXMIO (dev, id)) {
                printf ("\nInput new baudrate (current %d:%dbps) 0..4 = ", n, baudlist[n]);
                if (InputNum (&n)) {
                  if ((n >= 0 && n <= 4) /*|| (n == 6)*/) {
                    if (DX2_WriteByteData (dev, id, _ADDR_Baudrate, n, &err)) {
                      if (DX2_SetBaudrate (dev, baudlist[n])) {
                        baud = baudlist[n];
                        printf ("OK\n");
                      } else printf ("NG\n");
                    } else printf ("NG\n");
                  } else printf ("NG\n");
                } else printf ("NG\n");
              } else printf ("NG\n");
              break;
            case 'N': // NVM書き込み
              printf("\n>Write NVM Data\n");
              if (AreYouDXMIO (dev, id)) {
                if (DX2_WriteByteData (dev, id, _ADDR_WriteNVM, 1, &err)) printf ("OK\n");
                else printf ("NG\n");
              } else printf ("NG\n");
              break;
            case 'R': // 出荷時設定に戻す
              printf("\n>Factory Reset\n");
              if (DX2_Reset (dev, id, &err)) {
                printf ("OK\n");
                id = 200;
                baud = 1000000;
                DX2_SetBaudrate (dev, baud);
              } else printf ("NG\n");
              break;
            case 'L': // LEDの状態を反転
              printf("\n>Set/Reset LED\n");
              if (AreYouDXMIO (dev, id)) {
                if (DX2_ReadByteData (dev, id, _ADDR_LED, &led, &err)) {
                  led ^= 1;
                  if (DX2_WriteByteData (dev, id, _ADDR_LED, led, &err)) {
                    printf ("LED=%d OK\n", led);
                  } else printf ("NG\n");
                } else printf ("NG\n");
              } else printf ("NG\n");
              break;
            case 'P': // GPIOのモード変更 (ここでは簡便のため全端子共通)
              printf("\n>Set pin config");
              if (AreYouDXMIO (dev, id)) {
                if (DX2_ReadByteData (dev, id, _ADDR_PinCfg, &b, &err)) {
                  printf ("\nInput new mode (current %d) 0..5 = ", b);
                  if (InputNum (&n)) {
                    if (n >= 0 && n <= 5) {
                      for (int i = 0; i < 12; i++) DX2_WriteByteData (dev, id, _ADDR_PinCfg + i, n, &err);
                    } else printf ("NG\n");
                  } else printf ("NG\n");
                } else printf ("NG\n");
              } else printf ("NG\n");
              break;
            case 'O': // IMUの動作モード変更
              printf("\n>Set IMU OPMode");
              if (AreYouDXMIO (dev, id)) {
                if (DX2_ReadByteData (dev, id, _ADDR_IMU_OprMode, &b, &err)) {
                  printf ("\nInput new OPMode (current %d) 0..12 = ", b);
                  if (InputNum (&n)) {
                    if (n >= 0 && n <= 12) {
                      if (DX2_WriteByteData (dev, id, _ADDR_IMU_OprMode, n, &err)) printf ("OK\n");
                      else printf ("NG\n");
                    } else printf ("NG\n");
                  } else printf ("NG\n");
                } else printf ("NG\n");
              } else printf ("NG\n");
              break;
            case 'T':
              if (AreYouDXMIO (dev, id)) {
                cls ();
                while (!kbhit ()) {
                  // DXMIOの全コントロールテーブルを一括読み出し
                  if (DX2_ReadBlockData (dev, id, 0, (uint8_t *)&table, sizeof (TE097BCtrlTable), &err)) {
                    // 測定値を数値表示
                    gotoxy (0, 0);
                    printf (
                      "IMU[%8.3f %8.3f %8.3f] GPIO[%03x] ADC[%03x %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x %03x]",
                      table.IMU.heading / 16.0, table.IMU.roll / 16.0, table.IMU.pitch / 16.0,
                      table.GPIO_IN,
                      table.ADV[0], table.ADV[1], table.ADV[2], table.ADV[3], table.ADV[4], table.ADV[5], table.ADV[6], table.ADV[7], table.ADV[8], table.ADV[9], table.ADV[10], table.ADV[11]
                    );
                    // 一部値をグラフィック表示
                    Draw (hwd, hdc, pen, brush, &table, &prevx, prevy, x);
                    x++;
                  } else
                    break;
                }
                while (kbhit ())_getch ();
              }
              break;
            case 'M':
              if (AreYouDXMIO (dev, id)) {
                cls ();
                while (!kbhit ()) {
                  if (DX2_ReadBlockData (dev, id, 0, (uint8_t *)&table, sizeof (TE097BCtrlTable), &err)) {
                    gotoxy (0, 0);
                    printf ("  0: $%04X $%08X $%02X\n", table.ModelNumber, table.ModelInfomation, table.FirmwareVersion);
                    printf ("  7: ID %3d\n", table.ID);
                    printf ("  8: BAUD %d\n", table.Baudrate);
                    printf ("  9: %d\n", table.WriteNVM);
                    printf (" 10: LED %d\n", table.LED);
                    printf (" 11: %d\n", table.Terminator);
                    printf (" 12: CFG %2d %2d %2d %2d %2d %2d %2d %2d %2d %2d %2d %2d\n", table.PinCfg[0], table.PinCfg[1], table.PinCfg[2], table.PinCfg[3], table.PinCfg[4], table.PinCfg[5], table.PinCfg[6], table.PinCfg[7], table.PinCfg[8], table.PinCfg[9], table.PinCfg[10], table.PinCfg[11]);
                    printf (" 24: FREQ %5d\n", table.PWMFrequency);
                    printf (" 26: PWM %5d %5d %5d %5d %5d %5d\n", table.PWMDuty[0], table.PWMDuty[1], table.PWMDuty[2], table.PWMDuty[3], table.PWMDuty[4], table.PWMDuty[5]);
                    printf (" 48: CAP %5d %5d %5d %5d\n", table.Capture[0], table.Capture[1], table.Capture[2], table.Capture[3]);
                    printf (" 56: OUT/IN $%03X $%03X\n", table.GPIO_OUT, table.GPIO_IN);
                    printf (" 60: ADC $%03X $%03X $%03X $%03X $%03X $%03X $%03X $%03X $%03X $%03X $%03X $%03X\n", table.ADV[0], table.ADV[1], table.ADV[2], table.ADV[3], table.ADV[4], table.ADV[5], table.ADV[6], table.ADV[7], table.ADV[8], table.ADV[9], table.ADV[10], table.ADV[11]);
                    printf ("102: DAC $%03X\n", table.DAV[0]);
                    printf ("108: IMU OP/AXIS %2d %d\n",table.IMU_OprMode, table.IMU_AxisPlace);
                    printf ("110:  ACC %6d %6d %6d\n", table.IMU.acc_x, table.IMU.acc_y, table.IMU.acc_z);
                    printf ("116:  MAG %6d %6d %6d\n", table.IMU.mag_x, table.IMU.mag_y ,table.IMU.mag_z);
                    printf ("122:  GYRO %6d %6d %6d\n", table.IMU.gyro_x, table.IMU.gyro_y, table.IMU.gyro_z);
                    printf ("128:  HEAD %6d %6d %6d\n", table.IMU.heading, table.IMU.roll, table.IMU.pitch);
                    printf ("134:  QT %6d %6d %6d %6d\n", table.IMU.q_w, table.IMU.q_x, table.IMU.q_y, table.IMU.q_z);
                    printf ("142:  LIA %6d %6d %6d\n", table.IMU.lia_x, table.IMU.lia_y, table.IMU.lia_z);
                    printf ("148:  GRV %6d %6d %6d\n", table.IMU.grv_x, table.IMU.grv_y, table.IMU.grv_z);
                    printf ("154:  TMP %4d\n", table.IMU.temp);
                    printf ("155:  STAT/ERR %3d %3d\n", table.IMU_stat, table.IMU_err);
//                    Draw (hwd, hdc, pen, brush, &table, &prevx, prevy, x);
//                    x++;
                  } else break;
                }
                while (kbhit ())_getch ();
              }
              break;
            case '?':
              printf("\n>Command List");
              printf(
                "\n [s] Full Scan [i] Change host ID [b] Change host baudrate\n"
                " [I] Change DXMIO ID [B] Change DXMIO baudrate\n"
                " [N] Writing NVM [R] Restore factory defaults\n"
                " [L] Reverses LED status [P] GPIO mode change\n"
                " [O] Change of IMU operation mode\n"
                " [T] Trend Graph [M] Monitor all control tables\n"
                " [q/Q] Quit\n"
               );
              break;
            case 'q': // 終了
            case 'Q':
              term = true;
              break;
            default:
              for (n = 0; (n < 9) && (baudlist[n] != baud); n++) ;
              printf("\n>Host ID:%d, Host Baudrate:%d(%dbps)", id, n, baud);
              printf("\n>Ping = %s\n", DX2_Ping (dev, id, &err) ? "OK" : "NG");
              break;
          }
        } else
          Sleep (10);
      }

      // ポートクローズ
      DX2_ClosePort (dev);
    } else printf ("Failed to open %s\n", comname);

    // GDIオブジェクトの破棄
    DeleteObject (pen[0]);
    DeleteObject (pen[1]);
    DeleteObject (pen[2]);
    DeleteObject (pen[3]);
    DeleteObject (brush[0]);
    DeleteObject (brush[1]);
    ReleaseDC(hwd, hdc);

    // DX2LIBをアンロード
    UnloadDLL ();
  } else
    printf ("Fail to load DLL!\n");

  printf ("Fin\n");
}
