#include <stdio.h>
#include <string.h>
#include <conio.h>

#include <windows.h>
#include <fcntl.h>
#include <stdio.h>
#include <io.h>

#define _DYNAMICLOAD
#include "icslib.h"

// }N`
#define _COMPORT   (3)      // |[gԍ
#define _BAUDRATE  (115200) // {[[g[bps]
#define _TARGETID  (1)      // ΏۂƂȂID

TDeviceID   hdev;

typedef struct {
  int16_t acc_x;    // Acceleration Data
  int16_t acc_y;
  int16_t acc_z;
  int16_t mag_x;    // Magnetometer Data
  int16_t mag_y;
  int16_t mag_z;
  int16_t gyro_x;   // Gyroscope Data
  int16_t gyro_y;
  int16_t gyro_z;
  int16_t heading;  // Euler Data
  int16_t roll;
  int16_t pitch;
  int16_t q_w;      // Quaternion Data
  int16_t q_x;
  int16_t q_y;
  int16_t q_z;
  int16_t lia_x;    // Linear Acceleration Data
  int16_t lia_y;
  int16_t lia_z;
  int16_t grv_x;    // Gravity Vector Data
  int16_t grv_y;
  int16_t grv_z;
  int8_t  temp;     // Temperature
} __attribute__ ((packed)) Tbno055;

bool InputNum (int32_t *num) {
  char buf[100];
  fgets(buf, sizeof(buf), stdin);
  rewind(stdin);
  return (strlen(buf) && sscanf(buf, "%d", num) == 1);
}

bool EditComAndBaudAndTarget (char *comport, uint32_t *comn, uint32_t *baud, uint8_t *id) {
  bool result = true;

  int32_t n;

  printf ("Input comport num (default %d) = ", _COMPORT);
  if (!InputNum (&n)) { n = _COMPORT; result = false; }
  if (n >= 1 && n <= 254) *comn = n; else *comn = _COMPORT;
  sprintf(comport, "\\\\.\\COM%d", *comn);

  printf ("Input baudrate (default %dbps) = ", _BAUDRATE);
  if (!InputNum (&n)) { n = _BAUDRATE; result = false; }
  if (n >= 9600 && n <= 2000000) *baud = n; else *baud = _BAUDRATE;

  printf ("Input target id (default %d) = ", _TARGETID);
  if (!InputNum (&n)) { n = _TARGETID; result = false; }
  if (n >= 1 && n < 0xfe) *id = n; else *id = _TARGETID;

  return result;
}

void icsps (uint8_t *cc, int len) {
  ics_puts (hdev, cc, len);
  printf("TX:");
  for (int i=0;i<len;i++) printf("%02X%c", cc[i],i!=len-1?' ':'\n');
}

void icsgs (uint8_t *cc, int len, int to) {
  int rl;
  ics_gets (hdev, cc, len, &rl, to);
  printf("RX[%d]:",rl);
  for (int i=0;i<rl;i++) printf("%02X%c", cc[i],i!=len-1?' ':'\n');
}


bool ics_set_pos (uint8_t id, uint16_t gpos, uint16_t *ppos) {
  if (id > 31) return false;
  if (gpos > 11500) gpos = 11500;
  else if (gpos < 3500) gpos = 3500;
  uint8_t t[] = { 0x80 | id, (gpos >> 7) & 0x7f, gpos & 0x7f };
  ics_purge (hdev);
  ics_puts (hdev, t, sizeof (t));
  uint8_t r[10];
  int l;
  bool result = ics_gets (hdev, r, 3, &l, 10.0);
  if (result && (l == 3) && r[1] == (t[0] & 0x7f) && ppos != NULL) *ppos = (r[1] << 7) | r[2];
  return result;
}

bool ics_get_pos (uint8_t id, uint16_t *ppos) {
  if (id > 31 || ppos == NULL) return false;
  uint8_t t[] = { 0xa0 | id, 0x5 };
  ics_purge (hdev);
  ics_puts (hdev, t, sizeof (t));
  uint8_t r[10];
  int l;
  bool result = ics_gets (hdev, r, 4, &l, 10.0);
  if (result && (l == 4) && r[1] == 0x05) *ppos = (r[2] << 7) | r[3];
  return result;
}

bool ics_set_gp (uint8_t id, uint8_t addr, uint8_t len, uint8_t *dat) {
  if (addr > 127 || len > 127 || dat == NULL) return false;
  uint8_t t[300] = { 0xc0 | id, 0x7f, addr, len };
  for (int i = 0; i < len; i++) {
    t[4 + i * 2] = dat[i] >> 4;
    t[4 + i * 2 + 1] = dat[i] & 0xf;
  }
  ics_purge (hdev);
  ics_puts (hdev, t, 4 + len * 2);
  uint8_t r[10];
  int l;
  if (ics_gets (hdev, r, 4, &l, 10.0)) {
    return ((l == 4) && (t[0] & 0x7f) == r[0]) && (t[1] == r[1]) && (t[2] == r[2]) && (t[3] == r[3]);
  }
  return false;
}

bool ics_get_gp (uint8_t id, uint8_t addr, uint8_t len, uint8_t *dat) {
  if (addr > 127 || len > 127 || dat == NULL) return false;
  uint8_t t[] = { 0xa0 | id, 0x7f, addr, len };
  ics_purge (hdev);
  ics_puts (hdev, t, sizeof (t));
  uint8_t r[300];
  int l;
  if (ics_gets (hdev, r, len * 2 + 4, &l, 10.0)) {
    if ((l == len * 2 + 4) && ((t[0] & 0x7f) == r[0]) && (t[1] == r[1]) && (t[2] == r[2]) && (t[3] == r[3])) {
//printf("addr=%d sz=%d rlen=%d <%02X,%02X>\n", addr, len, l, r[4], r[5]);
      for (int i = 0; i < len; i++) {
        if (r[i * 2 + 4] > 0x10 || r[i * 2 + 1 + 4] > 0x10) return false;
        dat[i] = (r[i * 2 + 4] << 4) | r[i * 2 + 1 + 4];
      }
      return true;
    }
  }
  return false;
}

bool ics_get_eep (uint8_t id, uint8_t *dat) {
  uint8_t r[100];
  int l;
  if (dat == NULL) return false;
  uint8_t t[] = {0xA0 | id, 0x00};
  ics_purge (hdev);
  ics_puts (hdev, t, sizeof (t));
  if (ics_gets (hdev, r, 66, &l, 50.0)) {
    if ((l == 66) && ((t[0] & 0x7f) == r[0]) && (t[1] == r[1])) {
      for (int i = 0; i < 32; i++) {
        if (r[i * 2 + 2] > 0x10 || r[i * 2 + 2 + 1] > 0x10) return false;
        dat[i] = (r[i * 2 + 2] << 4) | r[i * 2 + 2 + 1];
      }
      return true;
    }
  }
  return false;
}

bool enable_virtual_terminal_processing(FILE *stream) {
    HANDLE handle = (HANDLE)_get_osfhandle(_fileno(stream));
    DWORD mode = 0;
    if (!GetConsoleMode(handle, &mode)) {
        // s
        return false;
    }
    if (!SetConsoleMode(handle, mode | ENABLE_VIRTUAL_TERMINAL_PROCESSING)) {
        // s
        // ÂWindowsR\[̏ꍇ GetLastError() == ERROR_INVALID_PARAMETER
        return false;
    }
    return true;
}


#define cls()           printf ("\x1b[2J");
#define gotoxy(xx, yy)  printf ("\x1b[%d;%dH", yy + 1, xx + 1);

uint32_t baudlist[]={115200,625000,1250000};


uint8_t krsid  = 0;


void main(void){
  char comname[20];
  uint8_t dd[200];
  uint32_t comnum, baud;
  uint8_t id;
  Tbno055 bno;

  int baudind = 0;
  uint32_t baudrate = 115200;

  enable_virtual_terminal_processing(stdout);

  EditComAndBaudAndTarget (comname, &comnum, &baud, &id);

  if (LoadDLL ()) {
    // |[gI[v
    if ((hdev = ICS_OpenPort (comname, baud)) != 0) {
      printf("Open success\n");
      ICS_SetTimeOutOffset(hdev, 50);
      ICS_SetEchoCancel (false);

      ics_purge (hdev);

      uint16_t pos = 7500;
      int di = 50;
      uint16_t err;

      bool mes = false;
      bool ppp = false;
      printf ("id:%d(%d)> ", id ,baudind);
      while (1) {
        if (mes) {
  //        if (!ics_get_gp (id, 0, 121, dd)) printf (" ERR");
          if (!ics_get_gp (id, 74, sizeof(Tbno055), dd)) printf (" ERR");
          else {
            memcpy (&bno, &dd[0], sizeof(Tbno055));
#if 1
            gotoxy(0, 0);
            printf ("acc:%6d,%6d,%6d\n", bno.acc_x, bno.acc_y, bno.acc_z);
            printf ("mag:%6d,%6d,%6d\n", bno.mag_x, bno.mag_y, bno.mag_z);
            printf ("gyr:%6d,%6d,%6d\n", bno.gyro_x, bno.gyro_y, bno.gyro_z);
            printf ("eul:%6d,%6d,%6d\n", bno.heading, bno.roll, bno.pitch);
            printf ("qua:%6d,%6d,%6d,%6d\n", bno.q_w, bno.q_x, bno.q_y, bno.q_z);
            printf ("lia:%6d,%6d,%6d\n", bno.lia_x, bno.lia_y, bno.lia_z);
            printf ("grv:%6d,%6d,%6d\n", bno.grv_x, bno.grv_y, bno.grv_z);
            printf ("tmp;   %3d\n", bno.temp);
#endif
          }
        }
        if (ppp) {
          pos += di;
          if (pos > 11500) { pos = 11500; di = -50; }
          else if (pos < 3500) { pos = 3500; di = 50; }
          printf("pos(%d:pos=%d)     \n",ics_set_pos (krsid, pos, NULL),pos);
        }

        if (kbhit ()) {
          uint8_t b;
          int32_t d;
          switch (_getch ()) {

            case 'I':
              krsid++;
              if (krsid > 31) krsid = 0;
              printf ("krsid=%d\n", krsid);
            case 'p':
              ppp = !ppp;
              break;

            case 's': // 

              printf("Scan DXMIO\n");
              printf("Start baud index(0..2)=");
              if (InputNum (&d)) {
                if (d<0) d=0; else if (d>2) d=2;
                for (int i = d; i <= 2 && !kbhit(); i++) {
                  ICS_SetBaudrate (hdev, baudlist[i]);
                  for (int j = 0; j <= 31 && !kbhit(); j++) {
                    printf("Ping baud:%dbps id:%d    \r",baudlist[i],j);
                    if (ics_get_eep (j, dd)) {
                      Sleep (50);
                      if (ics_get_eep (j, dd)) {
                        Sleep (50);
                        if (ics_get_eep (j, dd)) {
                          Sleep (50);
                          if (ics_get_gp (j, 4, 1, &b)) {
                            baudind = b;
                            id = j;
                            baudrate = baudlist[baudind];
                            printf("Detect id:%d baud:%d(%dbps)        \n", j, baudind, baudrate);
                            goto find;
                          }
                        }
                      }
                    }
                  }
                  Sleep (10);
                }
                baudind = 0;
                baudrate = baudlist[baudind];
                ICS_SetBaudrate (hdev, baudrate);
find:
                if (kbhit ()) _getch();
                printf("\n");
              }

              break;
            case 'b': // {[[gύX
              printf("Change Baud\n");
              if (ics_get_gp (id, 4, 1, &b)) {
                printf("%d(%dbps) -> ", b, baudlist[b]);
                if (InputNum (&d)) {
                  if (d < 0) d = 0; else if (d > 2) d = 2;
                  b = d;
                  if (ics_set_gp (id, 4, 1, &b)) {
                    printf("Set Baud OK\n");
                    baudind = b;
                    baudrate = baudlist[baudind];
                    printf("Update baud %d(%dbps)\n",baudind, baudrate);
                    ICS_SetBaudrate (hdev, baudrate);
                  } else {
                    printf("Set Baud NG\n");
                  }
                }
              } else {
                printf ("Get Baud ERR \n");
                printf("Change host baud %d -> ", baudind);
                if (InputNum (&d)) {
                  if (d < 0) d = 0; else if (d > 2) d = 2;
                  baudind = d;
                  baudrate = baudlist[baudind];
                  printf("%d(%dbps)\n",baudind, baudrate);
                  ICS_SetBaudrate (hdev, baudrate);
                }
              }

              break;
            case 'n': // ID,BAUDRATE,TERMINATOR NVMۑ
              printf("Write NVM ");
              b = 1;
              if (!ics_set_gp (id, 5, 1, &b)) printf(" ERR \n");
              else printf("OK\n");
              break;
            case 'm': // IMU j^ON/OFF
              cls ();
              mes = !mes;
              break;
            case 'i': // IDύX
              printf("Change ID\n");
              if (ics_get_gp (id, 3, 1, &b)) {
                printf("%d -> ", b);
                if (InputNum (&d)) {
                  if (d < 0) d = 0; else if (d > 31) d = 31;
                  b = d;
                  if (!ics_set_gp (id, 3, 1, &b)) printf(" ERR \n");
                  else printf("OK\n");
                  id = b;
                }
              } else {
                printf (" ERR \n");
                printf("Change target id %d -> ", id);
                if (InputNum (&d)) {
                  if (d < 0) d = 0; else if (d > 31) d = 31;
                  printf ("%d", d);
                  id = d;
                }
              }
              break;
            case 'l': // LED ON/OFF
              printf("Toggle LED\n");
              if (ics_get_gp (id, 6, 1, &b)) {
                b ^= 1;
                if (!ics_set_gp (id, 6, 1, &b)) printf (" ERR \n");
                else printf("%s\n", b == 1 ? "ON":"OFF");
              } else printf (" ERR \n");
              break;
            case 't': // TERMINATOR ON/OFF
              printf("Toggle Terminator\n");
              if (ics_get_gp (id, 7, 1, dd)) {
                b ^= 1;
                if (!ics_set_gp (id, 7, 1, dd)) printf (" ERR \n");
                else printf("%s\n", b == 1 ? "ON":"OFF");
              } else printf (" ERR \n");
              break;
            case '!': // I
            case 0x1b:
              printf("Fin\n");
              return ;
              break;
            default:
              printf("\n");
              break;
          }
          printf ("id:%d(%d)> ", id ,baudind);
        }
      }

      // |[gN[Y
      ICS_ClosePort (hdev);
    } else {
      printf ("\nOpenPort Error");
    }
    UnloadDLL ();
  }

  printf ("\nFin");
}
